@@ -25,28 +25,59 @@ class RippleApiError(Exception):
         return '%s. %s' % (self.error, self.message)
 
 
-def call_api(data, api_user=None, api_password=None):
-    if api_user is None:
-        api_user = settings.RIPPLE_API_USER
-    if api_password is None:
-        api_password = settings.RIPPLE_API_PASSWORD
-    auth = (api_user, api_password) if api_user or api_password else None
-    response = requests.post(settings.RIPPLE_API_URL, json.dumps(data), auth=auth, verify=False)
-
-    try:
-        result = response.json()['result']
-    except ValueError:
-        raise RippleApiError('Error', '', response.text)
-
-    if not 'error' in result:
-        return result
+def call_api(data, server_url=None, api_user=None, api_password=None):
+    error = None
+    if server_url and not (api_user or api_password):
+        servers = filter(
+            lambda item: item.get('RIPPLE_API_URL', '') == server_url,
+            settings.RIPPLE_API_DATA
+        )
+        servers = servers or [{'RIPPLE_API_URL': server_url}]
+    elif server_url and (api_user or api_password):
+        servers = [
+            {
+                'RIPPLE_API_URL': server_url,
+                'RIPPLE_API_USER': api_user,
+                'RIPPLE_API_PASSWORD': api_password,
+            }
+        ]
     else:
-        raise RippleApiError(result['error'], result['error_code'], result['error_message'])
+        servers = settings.RIPPLE_API_DATA
+
+    for server_config in servers:
+        url = server_config.get('RIPPLE_API_URL', '')
+        user = server_config.get('RIPPLE_API_USER', '')
+        pwd = server_config.get('RIPPLE_API_PASSWORD', '')
+        auth = (user, pwd) if user or pwd else None
+        try:
+            response = requests.post(
+                url, json.dumps(data), auth=auth, verify=False)
+        except requests.ConnectionError, e:
+            error = e
+            continue
+
+        try:
+            result = response.json()['result']
+        except ValueError:
+            error = RippleApiError('Error', '', response.text)
+            continue
+
+        if not 'error' in result:
+            return result
+        else:
+            error = RippleApiError(
+                result['error'],
+                result.get('error_code', 'no_code'),
+                result.get('error_message', 'no_message'),
+            )
+            continue
+    raise error
 
 
 def account_tx(
-        account, ledger_index_min=-1, ledger_index_max=-1, binary=False, forward=False, limit=None,
-        marker=None, api_user=None, api_password=None):
+        account, ledger_index_min=-1, ledger_index_max=-1, binary=False,
+        forward=False, limit=None, marker=None,
+        server_url=None, api_user=None, api_password=None):
     """
     Fetch a list of transactions that applied to this account.
 
@@ -82,10 +113,10 @@ def account_tx(
     if marker:
         data['params'][0]['marker'] = marker
 
-    return call_api(data, api_user, api_password)
+    return call_api(data, server_url, api_user, api_password)
 
 
-def tx(transaction_id, api_user=None, api_password=None):
+def tx(transaction_id, server_url=None, api_user=None, api_password=None):
     """
     Return information about a transaction.
 
@@ -97,12 +128,13 @@ def tx(transaction_id, api_user=None, api_password=None):
     data = {"method": "tx",
             "params": [{'transaction': transaction_id}]}
 
-    return call_api(data, api_user, api_password)
+    return call_api(data, server_url, api_user, api_password)
 
 
 def sign(
-        account, secret, destination, amount, send_max=None, paths=None, flags=None,
-        destination_tag=None, transaction_type='Payment', api_user=None, api_password=None):
+        account, secret, destination, amount, send_max=None, paths=None,
+        flags=None, destination_tag=None, transaction_type='Payment',
+        server_url=None, api_user=None, api_password=None):
     """
     After you've created a transaction it must be cryptographically signed using the secret belonging to the owner of
     the sending address. Signing a transaction prior to submission allows you to maintain closer control over
@@ -159,10 +191,10 @@ def sign(
     if destination_tag:
         data['params'][0]['tx_json']['DestinationTag'] = destination_tag
 
-    return call_api(data, api_user, api_password)
+    return call_api(data, server_url, api_user, api_password)
 
 
-def submit(tx_blob, api_user=None, api_password=None):
+def submit(tx_blob, server_url=None, api_user=None, api_password=None):
     """
     Submits a transaction to the network.
 
@@ -177,4 +209,4 @@ def submit(tx_blob, api_user=None, api_password=None):
         "params": [{
             "tx_blob": tx_blob}]}
 
-    return call_api(data, api_user, api_password)
+    return call_api(data, server_url, api_user, api_password)
